@@ -31,6 +31,7 @@
 //#include <hildon/hildon.h>
 
 #define G_DEBUG_DISABLE 1
+#define VERSION "0.0.3"
 
 #define BUTTON_WIDTH 80
 #define BUTTON_HIGHT 60
@@ -44,7 +45,6 @@
 #define HILDON_IM_FCITX_FKB_GET_PRIVATE(obj) \
         (G_TYPE_INSTANCE_GET_PRIVATE ((obj), HILDON_IM_FCITX_FKB_TYPE,\
                                       HildonIMFcitxFKBPrivate))
-static GtkWidget *window, *edit;
 /* ***** LAYOUTS ***** */
 
 static const gchar* alpha_lower_0[] = {"q", "w", "e", "r", "t", "y", "u", "i", "o", "p", NULL};
@@ -89,6 +89,8 @@ static const gchar** special_cn[] = {special_cn_0,
                                      special_cn_1,
                                      special_cn_2,
                                      special_cn_3, NULL};
+static osso_context_t *osso_context;
+static gboolean chinese_mode;
 
 typedef struct
 {
@@ -113,7 +115,6 @@ typedef struct
   gint timeout_handler;
   gint deleted;
 
-  gboolean chinese_mode;
   gboolean return_close;
 
   GtkWidget *vbox;
@@ -124,7 +125,6 @@ typedef struct
   GtkWidget *return_button;
   GtkWidget *delete_button;
   GtkWidget *mode_button;
-  const gchar*** keyboard_layout;
   GtkWidget* buttons[4][10];
 }
 HildonIMFcitxFKBPrivate;
@@ -205,7 +205,7 @@ static void hildon_im_fcitx_fkb_init       (HildonIMFcitxFKB *self);
 static void populate_window (HildonIMFcitxFKB *self);
 static void button_update ( HildonIMFcitxFKBPrivate *priv, const gchar** keyboard_layout[]);
 static void close_fkb (GtkWidget *button, gpointer user_data);
-
+static void power_off_save_data();
 /*
  * Module functions
  */
@@ -253,6 +253,9 @@ module_init(GTypeModule *module)
                               HILDON_IM_FCITX_FKB_TYPE,
                               HILDON_IM_TYPE_PLUGIN,
                               &plugin_info);
+  chinese_mode = TRUE;
+  osso_context = osso_initialize("org.maemo.fcitm", VERSION, TRUE, NULL);
+  osso_hw_set_event_cb(osso_context, NULL, power_off_save_data, NULL);
 }
 
 /*
@@ -520,7 +523,6 @@ hildon_im_fcitx_fkb_enable (HildonIMPlugin *plugin,gboolean init)
   g_return_if_fail (HILDON_IS_IM_FCITX_FKB (plugin));
   self = HILDON_IM_FCITX_FKB(plugin);
   priv = HILDON_IM_FCITX_FKB_GET_PRIVATE(self);
-  priv->window = window;
   window_is_new = (priv->window == NULL);
   /*  Send  g_debug() to the file log */
 #ifndef G_DEBUG_DISABLE
@@ -532,7 +534,6 @@ hildon_im_fcitx_fkb_enable (HildonIMPlugin *plugin,gboolean init)
     gtk_window_set_type_hint(GTK_WINDOW(priv->window), GDK_WINDOW_TYPE_HINT_DIALOG | GDK_WINDOW_TYPE_HINT_NOTIFICATION);
     gtk_window_set_default_size(GTK_WINDOW(priv->window),800,480);
     gtk_window_set_decorated (GTK_WINDOW (priv->window), FALSE);
-    window = priv->window;
     /*g_signal_connect(priv->window, "destroy", G_CALLBACK(close_win), self);*/
   }
   /*
@@ -544,7 +545,6 @@ hildon_im_fcitx_fkb_enable (HildonIMPlugin *plugin,gboolean init)
                                           HILDON_IM_CONTEXT_REQUEST_SURROUNDING_FULL);
   hildon_im_ui_play_sound(priv->ui, HILDON_IM_FINGER_TRIGGER_SOUND);
   priv->saved_offset = 0;
-  gtk_widget_show(GTK_WIDGET(priv->window));
 
   gdk_window_set_transient_for(GTK_WIDGET(priv->window)->window,
                                gtk_widget_get_root_window(GTK_WIDGET(priv->window)));
@@ -555,12 +555,8 @@ hildon_im_fcitx_fkb_enable (HildonIMPlugin *plugin,gboolean init)
     gtk_widget_show_all(GTK_WIDGET(priv->window));
     gtk_widget_hide(GTK_WIDGET(priv->buttons[3][2]));/* Dont show in english mode*/
     PYInit();
-    osso_context_t *osso_context;
-    osso_context = osso_initialize("org.maemo.fcitm", "0.0.1", TRUE, NULL);
-    osso_hw_set_event_cb(osso_context, NULL, power_off_save_data, NULL);
-    priv->chinese_mode = TRUE;
+    gtk_widget_grab_focus(priv->text_view);
   }
-  gtk_widget_grab_focus(priv->text_view);
   priv->return_close = FALSE; /* Browser and Xterminal use return_key as close*/
 }
 
@@ -574,10 +570,7 @@ hildon_im_fcitx_fkb_disable (HildonIMPlugin *plugin)
   self = HILDON_IM_FCITX_FKB(plugin);
   priv = HILDON_IM_FCITX_FKB_GET_PRIVATE(self);
 
-  if (GTK_WIDGET_VISIBLE(GTK_WIDGET(priv->window)))
-  {
-    gtk_widget_hide(GTK_WIDGET(priv->window));
-  }
+  gtk_widget_destroy(priv->window);
 }
 /*static void
 hildon_im_fcitx_fkb_key_event(HildonIMPlugin *plugin,
@@ -601,7 +594,6 @@ hildon_im_fcitx_fkb_surrounding_received(HildonIMPlugin *plugin,
   self = HILDON_IM_FCITX_FKB(plugin);
   priv = HILDON_IM_FCITX_FKB_GET_PRIVATE(self);
 
-  priv->text_view = edit;
   buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(priv->text_view));
 
   /* I don't know why, but the view doesn't reflect the changes immediately */
@@ -719,7 +711,7 @@ text_button_clicked (GtkButton *button, gpointer user_data)
   priv = HILDON_IM_FCITX_FKB_GET_PRIVATE(self);
 
   text = gtk_button_get_label(button);
-  if(priv->chinese_mode){
+  if(chinese_mode){
     if(('a' <= *text && 'z' >= *text) || ',' == *text || '.' == *text){
       if(iPYInsertPoint == 0 && ',' == *text)
         text = "，";
@@ -869,14 +861,14 @@ switch_button_toggled (GtkButton *button,
   self = HILDON_IM_FCITX_FKB(user_data);
   priv = HILDON_IM_FCITX_FKB_GET_PRIVATE(self);
 
-  if (priv->chinese_mode)
+  if (chinese_mode)
   {
-    priv->chinese_mode = FALSE;
+    chinese_mode = FALSE;
     gtk_button_set_label (GTK_BUTTON(priv->mode_button),"英");
   }
   else
   {
-    priv->chinese_mode = TRUE;
+    chinese_mode = TRUE;
     gtk_button_set_label (GTK_BUTTON(priv->mode_button),"拼");
   }
 }
@@ -894,7 +886,7 @@ shift_symbols_buttons_toggled (GtkToggleButton *togglebutton,
 
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(priv->symbols_button)))
   {
-    if(iPYInsertPoint && priv->chinese_mode){
+    if(iPYInsertPoint && chinese_mode){
       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(priv->symbols_button), FALSE);
       PYGetCandWords(SM_NEXT);
       show_candwords(priv);
@@ -907,7 +899,7 @@ shift_symbols_buttons_toggled (GtkToggleButton *togglebutton,
     else
     {
       gtk_button_set_label(GTK_BUTTON(priv->space_button),"\t");
-      if(priv->chinese_mode)
+      if(chinese_mode)
         button_update(priv, special_cn);
       else
         button_update(priv, special_lower);
@@ -915,7 +907,7 @@ shift_symbols_buttons_toggled (GtkToggleButton *togglebutton,
   }else{
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(priv->shift_button)))
     {
-      if(iPYInsertPoint && priv->chinese_mode){
+      if(iPYInsertPoint && chinese_mode){
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(priv->shift_button), FALSE);
         PYGetCandWords(SM_PREV);
         show_candwords(priv);
@@ -944,7 +936,7 @@ return_button_clicked (GtkButton *button, gpointer user_data)
 
   delete_selection (self);
   buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(priv->text_view));
-  if(priv->chinese_mode && iPYInsertPoint){
+  if(chinese_mode && iPYInsertPoint){
     if (hildon_im_ui_get_commit_mode(priv->ui) == HILDON_IM_COMMIT_REDIRECT){
       hildon_im_ui_send_utf8(priv->ui, (const char*)(&strCodeInput));
       gtk_text_buffer_get_iter_at_mark(buffer, &iter,
@@ -1150,7 +1142,6 @@ populate_window (HildonIMFcitxFKB *self)
   gtk_text_view_set_right_margin(GTK_TEXT_VIEW(priv->text_view), 8);
 
   gtk_widget_modify_font(GTK_WIDGET(priv->text_view),font);
-  edit = priv->text_view;
   g_signal_connect(G_OBJECT(priv->text_view), "button-release-event",
                          G_CALLBACK(textview_button_release_cb), self);
 
@@ -1171,6 +1162,9 @@ populate_window (HildonIMFcitxFKB *self)
   gtk_widget_show_all (menu);
 
   parea = gtk_scrolled_window_new (NULL, NULL);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (parea),
+                                  GTK_POLICY_NEVER,
+                                  GTK_POLICY_AUTOMATIC);
   //parea = hildon_pannable_area_new();
   gtk_container_add (GTK_CONTAINER (parea), priv->text_view);
   gtk_container_add (GTK_CONTAINER (priv->vbox), parea);
@@ -1218,7 +1212,7 @@ populate_window (HildonIMFcitxFKB *self)
 
       /* Chinese_switch button */
 
-      priv->mode_button = gtk_button_new_with_label ("拼");
+      priv->mode_button = gtk_button_new_with_label (chinese_mode?"拼":"英");
       gtk_widget_set_size_request(priv->mode_button, BUTTON_WIDTH, BUTTON_HIGHT);
       gtk_box_pack_start(GTK_BOX(hbox), priv->mode_button, FALSE, FALSE, 0);
       g_signal_connect(priv->mode_button, "clicked", G_CALLBACK(switch_button_toggled), self);
